var Bridge = (function (index) {
'use strict';

var __assign = (window && window.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
function convertSnakeCase(key) {
    return key.replace(/_+([^_])/g, function (match, p1, offset, haystack) {
        return offset > 0 ? p1.toUpperCase() : match;
    });
}
// tslint:disable-next-line:no-any
function convertObject(obj) {
    // tslint:disable-next-line:no-any
    var result = {};
    for (var k in obj) {
        if (obj.hasOwnProperty(k)) {
            var value = obj[k];
            if (typeof value === 'object' && value !== null) {
                if (Array.isArray(value)) {
                    value = value.map(function (elt) {
                        if (typeof elt === 'object' && elt !== null) {
                            return convertObject(elt);
                        }
                        else {
                            return elt;
                        }
                    });
                }
                else {
                    value = convertObject(value);
                }
            }
            result[convertSnakeCase(k)] = value;
        }
    }
    return result;
}
// If we remove snake_case from `ExtensionDetails` and replace `ExtensionIconURLs` with
// `ExtensionIcons` this can be replaced with
// {
//     clientId: data.extension.id,
//     token: data.token,
//     ...data.extension,
// }
function makeExtension(details, token) {
    function convertIconURLs(obj) {
        return {
            square24: obj['24x24'],
            square100: obj['100x100'],
            discoverySplash: obj['300x200'],
        };
    }
    return convertObject(__assign({}, details, { clientId: details.id, iconUrls: details.icon_urls ? convertIconURLs(details.icon_urls) : undefined, token: token }));
}
// Converts PassedInAbilities (underscores) to ExtensionInstallationAbilities (camel cased)
function makeAbilities(abilities) {
    return convertObject(__assign({}, abilities));
}
/**
 * The Native<->JavaScript bridge for the extension coordinator.
 */
var Bridge = /** @class */ (function () {
    function Bridge() {
        this.extensionFrame = null;
        this.isLoaded = false;
        this.deferredIsVisible = null;
        this.callbacks = {};
        this.callbackCounter = 0;
        this.useBitsTransactions = {};
    }
    /**
     * Creates the extension frame and loads the extension.
     *
     * When the extension has loaded the `Native.onExtensionLoaded()` method will be invoked.
     *
     * This method must be invoked before calling any of the other methods.
     * @param mode The extension mode (either `"viewer"`, `"dashboard"`, or `"config"`).
     * @param language The language code (e.g. `"en"`).
     * @param locale The country code (e.g. `"US"`).
     * @param data The extension data to initialize the extension with.
     */
    Bridge.prototype.LoadExtension = function (mode, language, locale, data) {
        if (this.extensionFrame) {
            throw 'LoadExtension has already been called';
        }
        this.createExtensionFrame(mode, language, locale, data);
    };
    /**
     * Tells the extension coordinator to give the user's identity to the extension.
     *
     * **Important:** `LoadExtension()` must be invoked before calling this method.
     */
    Bridge.prototype.LinkUser = function () {
        if (this.extensionFrame) {
            this.extensionFrame.linkIdentity();
        }
        else {
            throw 'LinkUser cannot be called before LoadExtension';
        }
    };
    /**
     * Tells the extension coordinator to stop giving the user's identity to the extension.
     *
     * **Important:** `LoadExtension()` must be invoked before calling this method.
     */
    Bridge.prototype.UnlinkUser = function () {
        if (this.extensionFrame) {
            this.extensionFrame.unlinkIdentity();
        }
        else {
            throw 'UnlinkUser cannot be called before LoadExtension';
        }
    };
    /**
     * @deprecated Use `InvokeRequestCallback()` instead.
     */
    Bridge.prototype.InvokeFollowRequestCallback = function (id, result) {
        this.invokeCallback(id, result);
    };
    /**
     * @deprecated Use `InvokeRequestCallback()` instead.
     */
    Bridge.prototype.InvokeUseBitsRequestCallback = function (id, result) {
        this.invokeCallback(id, result);
    };
    /**
     * Invokes a model request callback.
     * @param id The callback identifier previously given to one of the `Native` request modal functions.
     * @param result The modal result. This must be the correct type for the callback.
     */
    Bridge.prototype.InvokeRequestCallback = function (id, result) {
        this.invokeCallback(id, result);
    };
    /**
     * Invokes a request callback with the default result (if any).
     *
     * If the given `ConfirmationRequest` defines a `defaultResult`, the registered callback for
     * `id` is invoked with that result. If it does not define a `defaultResult`, nothing happens.
     * @param id The callback identifier previously given to one of the `Native` request modal
     * functions (e.g. `Native.onFollowRequestModal()`).
     * @param confirmationRequest The confirmation request object given to that method (as a
     * deserialized JSON object rather than the serialized JSON string given to the `Native`
     * method).
     */
    Bridge.prototype.InvokeRequestCallbackWithDefaultResult = function (id, confirmationRequest) {
        if (confirmationRequest.defaultResult !== undefined) {
            this.invokeCallback(id, confirmationRequest.defaultResult);
        }
        else {
            this.unregisterCallback(id);
        }
    };
    /**
     * Posts a new context object to the extension.
     *
     * **Note:** You should not call this before `LoadExtension()`. It will not throw an error but it also won't
     * do anything.
     * @param context The new context object to give to the extension.
     */
    Bridge.prototype.PostContext = function (context) {
        index.extensionService.postContext(context);
    };
    /**
     * Marks the extension as either visible or invisible.
     *
     * An invisible extension does not receive any `onContext` callbacks and is discouraged from doing any work.
     *
     * **Note:** It's legal to call this before `LoadExtension()` but you are discouraged from doing so.
     * @param isVisible `true` if the extension is now visible, or `false` if it's hidden.
     */
    Bridge.prototype.SetVisible = function (isVisible) {
        if (this.isLoaded && this.extensionFrame) {
            this.extensionFrame.setVisible(isVisible);
        }
        else {
            this.deferredIsVisible = isVisible;
        }
    };
    Bridge.prototype.createExtensionFrame = function (mode, language, locale, data) {
        var _this = this;
        var enumMode = {
            config: index.ExtensionMode.Config,
            dashboard: index.ExtensionMode.Dashboard,
            viewer: index.ExtensionMode.Viewer,
        }[mode];
        index.extensionService.setEnvironmentOptions({ authToken: data.auth_token, env: data.env });
        this.extensionFrame = new index.ExtensionFrame({
            anchor: data.anchor,
            channelId: data.channel_id,
            configuration: data.configuration,
            loginId: data.user_id,
            dobbin: {
                trackEvent: function (eventName, properties, services) {
                    Native.onTrackEvent(eventName, JSON.stringify(properties), services);
                },
            },
            extension: makeExtension(data.extension, data.token),
            installationAbilities: makeAbilities(data.abilities),
            iframeClassName: 'mobile-iframe',
            language: language,
            locale: locale,
            mode: enumMode,
            platform: index.ExtensionPlatform.Mobile,
            parentElement: window.document.body,
            trackingProperties: {},
        });
        this.extensionFrame.on(index.ExtensionFrameEvents.IdentityLinked, function (isLinked, canLink) {
            Native.onIdentityLinked(isLinked, canLink);
        });
        this.extensionFrame.on(index.ExtensionFrameEvents.ExtensionFrameLoaded, function () {
            _this.isLoaded = true;
            if (_this.deferredIsVisible === false) {
                _this.extensionFrame.setVisible(_this.deferredIsVisible);
            }
            _this.deferredIsVisible = null;
            Native.onExtensionLoaded();
        });
        this.extensionFrame.on(index.ExtensionFrameEvents.RequestModal, function (confirmationRequest) {
            // tslint:disable-next-line:no-any
            function replacer(key, value) {
                if (value instanceof Error) {
                    // Error doesn't include message or name in its enumerable properties.
                    // This means the most useful information doesn't get serialized to JSON.
                    return __assign({ message: value.message, name: value.name }, value);
                }
                else {
                    return value;
                }
            }
            switch (confirmationRequest.action) {
                case index.FunctionAction.FollowAction:
                    var followCallbackId = _this.registerCallback(confirmationRequest.resultCallback);
                    Native.onFollowRequestModal(followCallbackId, JSON.stringify(confirmationRequest, replacer));
                    break;
                case index.FunctionAction.IdShareRequest:
                    Native.onIdShareRequestModal();
                    break;
                case index.FunctionAction.UseBitsPromptRequired:
                    var payload = confirmationRequest.options.payload;
                    if (payload && _this.useBitsTransactions.hasOwnProperty(payload.transactionId)) {
                        // This is a bits balance update to an existing transaction
                        Native.onUseBitsRequestBitsBalanceUpdated(payload.transactionId, payload.bitsBalance);
                    }
                    else {
                        // This is a new transaction
                        var transactionId_1 = payload ? payload.transactionId : null;
                        if (transactionId_1) {
                            _this.useBitsTransactions[transactionId_1] = null;
                        }
                        var useBitsCallbackId = _this.registerCallback(function (result) {
                            if (transactionId_1) {
                                delete _this.useBitsTransactions[transactionId_1];
                            }
                            confirmationRequest.resultCallback(result);
                        });
                        Native.onUseBitsRequestModal(useBitsCallbackId, JSON.stringify(confirmationRequest, replacer));
                    }
                    break;
                default:
                    var request = confirmationRequest;
                    if (request.resultCallback && request.defaultResult !== void 0) {
                        request.resultCallback(request.defaultResult);
                    }
                    break;
            }
        });
        this.extensionFrame.on(index.ExtensionFrameEvents.ShowBitsBalance, function () {
            Native.onShowBitsBalance();
        });
        this.extensionFrame.on(index.ExtensionFrameEvents.ShowUseBitsSuccess, function (bitsBalance) {
            Native.onShowUseBitsSuccess(bitsBalance);
        });
    };
    Bridge.prototype.registerCallback = function (callback) {
        this.callbackCounter += 1;
        this.callbacks[this.callbackCounter] = callback;
        return this.callbackCounter;
    };
    Bridge.prototype.invokeCallback = function (id, result) {
        var callback = this.callbacks[id];
        delete this.callbacks[id];
        if (callback !== void 0) {
            callback(result);
        }
    };
    Bridge.prototype.unregisterCallback = function (id) {
        delete this.callbacks[id];
    };
    return Bridge;
}());
// tslint:disable-next-line:no-default-export
var index$1 = new Bridge();

return index$1;

}(window['extension-coordinator']));
